{  8bpp PASM graphics library Doug Dingus 2011

Right now, this has no error checking or clipping, beware!  It will run past the screen buffer.

Intended for software clipping, usable display area 160 - clipping x 96 - clipping.

Typical would be 150 x 90 or so to allow for on-screen clipping.

This could use some speedups, and the line command is a WIP.

}

VAR
  BYTE cog

PUB start( parmptr )

  stop
  RESULT := COGNEW( @cogstart, parmptr)
  cog := RESULT + 1
  RETURN

PUB stop
   COGSTOP( cog~ - 1 )


DAT
                        ORG      0
cogstart
cmdwait                 rdlong  cmd, PAR    WZ            'get command block address into COG

              if_Z      jmp     #cmdwait

                        mov     A, PAR                    'read in all the arguments
                        add     A, #4                     'once
                        rdlong  arg1, A
                        add     A, #4
                        rdlong  arg2, A
                        add     A, #4
                        rdlong  arg3, A
                        add     A, #4
                        rdlong  arg4, A
                        
                        cmp     cmd, #01        WZ       'Is it clear screen?
              if_Z      jmp     #clear_screen            'YES!  Go and do it.

                        cmp     cmd, #02        WZ       'Is it bitplot?
              if_Z      jmp     #bitplot                 

                        cmp     cmd, #03        WZ       'Is it plot?
              if_Z      jmp     #plot
                        
              if_Z      cmp     cmd,  #04        WZ      'Is it fill?
                        jmp     #fill

              if_Z      cmp     cmd, #05        WZ      'Is it line?
                        jmp     #line                   'Not yet implemented

              if_Z      cmp     cmd, #06        WZ      'Is it setup?
                        jmp     #setup                  'Not yet implemented      

                        'fall through and reset to wait for valid command

cmd_done                wrlong  zero, PAR                'signal command done
                        jmp     #cmdwait


{       clear_screen = $01
        arg1 = screen address
        arg2 = size in bytes
        arg3 = clear color repeated in all four bytes
}
        

clear_screen            mov     A, arg1         'screen address
                        mov     B, arg2         'size in bytes
                        shr     B, #1           'convert size to longs
                        mov     C, #0           'init screen index
                        add     C, A
clr_loop                wrlong  arg3, C         'write color to screen
                        add     C, #4           'next screen address
                        djnz    B, #clr_loop
                        jmp     #cmd_done

{       fill = $04
        arg1 = screen address
        arg2 = position byte 0 = X, byte 1 = y
        arg3 = draw color
        arg4 = size byte 0 = X, byte 1 = y
}


fill                    call    #get_xyaddr             'Get fill start screen address 
                        
                        mov     bitplotline, #160       'prepare next line address
                        mov     X, arg4                 'Get fill region size
                        and     X, #$FF                 'Just X
                        mov     Y, arg4
                        shr     Y, #8                   'Just Y
                        and     Y, #$ff                                  
                        sub     bitplotline, X          'pre compute next line address index
                        mov     D, X                    'save X position for restore on each row

fill_next               nop
                        wrbyte  arg3, xyaddr            'Write the pixel
                        add     xyaddr, #1              'Advance to next screen buffer pixel address
                        djnz    X, #fill_next           'Draw the whole row
                        mov     X, D                    'Restore row address index for next pass
                        add     xyaddr, bitplotline     'Advance to next line on screen
                        djnz    Y, #fill_next          'Do all the rows in image

                        jmp     #cmd_done



{       bitplot = $02
        arg1 = screen address
        arg2 = position byte 0 = x, byte 1 = y
        arg3 = no draw color
        arg4 = address of source bitmap

        Source image assumed to be linear bitmap X*Y bytes in size, first two bytes containing that size
        
}

bitplot                 call    #get_xyaddr             'destination buffer address    arg2
                        mov     A, arg4                 'operate on source buffer address
                        mov     bitplotline, #160       'prepare next line address
                        rdbyte  X, A                    'Get X image size        
                        add     A, #1           
                        and     arg3, #$FF              'insure it's really just one byte color
                        rdbyte  Y, A                    'Get Y image size
                        sub     bitplotline, X          'pre compute next line address index
                        add     A, #1
                        mov     D, X                    'save X position
                        
                        
next_pixel              rdbyte  C, A                    'get first image pixel
                        cmp     C, arg3         WZ      'Do we skip that one?
              if_NZ     wrbyte  C, xyaddr               'Yes?  Don't write it, move on
                        add     xyaddr, #1              'Advance to next screen buffer pixel address
                        add     A, #1                   'Advance to next source image pixel address
                        djnz    X, #next_pixel          'Draw the whole row
                        mov     X, D                    'Restore row address index for next pass
                        add     xyaddr, bitplotline     'Advance to next line on screen
                        djnz    Y, #next_pixel          'Do all the rows in image

                        jmp     #cmd_done                        

                        

{       plot = $03
        arg1 = screen address
        arg2 = position, byte 0 = X, byte 1 = Y
        arg3 = dot color, byte 0
}                

plot                    call    #get_xyaddr

                        wrbyte  arg3, xyaddr
                        jmp     #cmd_done

{       get_xyaddr will compute screen address given x and y position
        arg1 = screen address
        arg2 = position, byte 0 = X, byte 1 = Y
        address left in xyaddr, A and B changed

        Written for 160 pixel screen, two shifts and two adds perform the multiply...
        This can work on other 8bpp screens, just adjust the shifts or include a more general
        and slower multiply.

        Values clipped to 8 bits x and y.  Most drivers do not exceed that anyway.
}

get_xyaddr              mov     A, arg2         'Get X size
                        mov     B, arg2         'Get Y size
                        and     A, #$FF         'X size only
                        shr     B, #8
                        and     B, #$FF         'Y size only

                        mov     xyaddr, B            'compute y address offset
                        shl     xyaddr, #7
                        add     xyaddr, arg1         'accumulate to screen address
                        shl     B, #5
                        add     xyaddr, B            'C now contains Y dot address
                        add     xyaddr, A            'C now contains complete dot address                  
get_xyaddr_ret          ret


{       Line = $05 stub for now.

}


line                    nop


                        jmp     #cmd_done





{       Setup = $06 may establish some parameters, stub for now.

}



setup                   nop


setup_ret               jmp     #cmd_done





DAT

zero          long      0

A             res 1
B             res 1

C             res 1
D             res 1
E             res 1
F             res 1

X             res 1
Y             res 1

xyaddr        res 1        
bitplotline   res 1             'This is a pre-computed address for drawing bitmaps

'cmdaddr       res 1
cmd           res 1     'current command, 0 = done / ready
arg1          res 1
arg2          res 1
arg3          res 1
arg4          res 1 